#!/usr/bin/python

#
# Copyright (C) 2008 by OpenMoko, Inc.
# Written by Willie Chen <willie_chen@openmoko.com>
# Wifi module is written by Tick <Tick@openmoko.com>
# All Rights Reserved
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#

import etk
import edje
import ecore.evas
import os
import re
import math

from expos import *

#etk.theme_widget_set_from_path("/usr/share/diversity-nav/default.edj")
LABELFONTSIZE="<font_size=16>"
ITEMFONTSIZE="<font_size=32>"
SUBFONTSIZE="<font_size=16>"

ee=ecore.evas.SoftwareX11(w=960, h=575)
canvas=ee.evas
ee.title="Exposure"

bg=canvas.Rectangle(color=(0, 0, 0, 255))
bg.size=canvas.size
bg.show()

edje_file="/usr/share/exposure/exposure.edj"
edj=edje.Edje(ee.evas, file=edje_file, group="demo")
edj.size=canvas.size
ee.data["edje"]=edj
edj.show()

embed1=etk.Embed(canvas)
embed2=etk.Embed(canvas)
embed3=etk.Embed(canvas)
embed4=etk.Embed(canvas)

def debug(msg):
	print msg
	pass
wifi_upstatus=()
wifi_uplist=()

def clicked(button):
	print "Clicked in item", button
	if re.match('\[\'GSM', str(button)):
		#GSM works
		debug('GSM')
		if gsm.check_onoff():
			gsm.power_off()
		else:
			gsm.power_on()
		lst.freeze()
		button[1]=label_text(button[0])
		lst.thaw()
	if re.match('\[\'WiFi', str(button)):
		#WiFi works
		global wifi_upstatus
		global wifi_uplist
		debug('WiFi')
		animator_move_object(embed1.object,0.5,0,0,-480,0)
		animator_move_object(embed3.object,0.5,480,0,0,0)
		wifi_upstatus = ecore.timer_add(2,update_status,lab)
		wifi_uplist = ecore.timer_add(5,update_list,lst_wifi,wifi)
	if re.match('\[\'Bluetooth', str(button)):
		#Bluetooth works
		debug('Bluetooth')
		if bt.check_onoff():
			bt.power_off()
		else:
			bt.power_on()
		lst.freeze()
		button[1]=label_text(button[0])
		lst.thaw()
	if re.match('\[\'GPS', str(button)):
		#GPS works
		debug('GPS')
		if gps.check_onoff():
			gps.power_off()
		else:
			gps.power_on()
		lst.freeze()
		button[1]=label_text(button[0])
		lst.thaw()
	if re.match('\[\'Brightness', str(button)):
		#Brightness works
		debug('Brightness')
		if display.get_brightness() > 40:
			display.set_brightness('31')
		else:
			display.set_brightness('63')
	if re.match('\[\'Display', str(button)):
		#Display works
		debug('Display')
        	animator_move_object(embed1.object,0.5,0,0,-480,0)
        	animator_move_object(embed2.object,0.5,480,0,0,0)
	if re.match('\[\'Sound', str(button)):
		#Sound Effects works
		debug('Sound Effects')
	if re.match('\[\'Ringtones', str(button)):
		#Ringtones works
		debug('Ringtones')
	if re.match('\[\'Vibrate', str(button)):
		#Vibrate works
		debug('Vibrate')

	if re.match('\[\'System', str(button)):
		#System Information works
		debug('System information')
        	animator_move_object(embed1.object,0.5,0,0,-480,0)
        	animator_move_object(embed4.object,0.5,480,0,0,0)
	
def	check_status(item):
	if re.match('GSM', item):
		if gsm.check_onoff():
			return 'on'
		else:
			return 'off'
	if re.match('Bluetooth', item):
		if bt.check_onoff():
			return 'on'
		else:
			return 'off'
	if re.match('GPS', item):
		if gps.check_onoff():
			return 'on'
		else:
			return 'off'
	else:
		return ''

def label_text(item):
    return "<font_size=48>"+item+"<font_size=24>"+'  '+check_status(item)

def click_back(button):
	global wifi_upstatus
	global wifi_uplist
	animator_move_object(embed1.object,0.5,-480,0,0,0)
	animator_move_object(embed3.object,0.5,0,0,480,0)
	wifi_upstatus.delete()
	wifi_uplist.delete()
	return True
def click_back1(button):
        animator_move_object(embed1.object,0.5,-480,0,0,0)
        animator_move_object(embed2.object,0.5,0,0,480,0)
        return True

def click_back2(button):
        animator_move_object(embed1.object,0.5,-480,0,0,0)
        animator_move_object(embed4.object,0.5,0,0,480,0)
        return True

class ObjectShift:
        def __init__(self,obj,finish_time,sx,sy,ex,ey):
                self.obj=obj
                self.sx=sx
                self.sy=sy
                self.ex=ex
                self.ey=ey
                self.start_time=ecore.time_get();
                self.finish_time=finish_time

def __animator_move_object(OS):
        elapse=ecore.time_get()-OS.start_time
        if elapse >= OS.finish_time:
                OS.obj.pos_set(OS.ex,OS.ey)
                del OS;
                return 0;
        ratio = 1 - (elapse/OS.finish_time);
        ratio = 1 - (ratio*ratio*ratio);
        x = OS.sx + ratio * (OS.ex - OS.sx);
        y = OS.sy + ratio * (OS.ey - OS.sy);
        OS.obj.pos_set(x,y)
        return 1

def animator_move_object(obj,finish_time, sx,sy,ex,ey):
        os = ObjectShift(obj,finish_time,sx,sy,ex,ey)
        ecore.animator_add(__animator_move_object,os)

###### Main Page #########
main_box = etk.VBox()
model = etk.ListModel()
ktr = etk.KineticTextRenderer(slot=1, click=clicked)
lst = etk.List(model, selectable=True, animated_changes=True,row_height=80,
columns=[(480,ktr,True)])

gps=gps()
bt=bt()
gsm=gsm()
display=display()
for item in ['GSM', 'WiFi', 'Bluetooth', 'GPS', 'Display', 'System Info']:
	lst.freeze()
	model.append([item, label_text(item)])
	lst.thaw()

main_box.append(lst, etk.VBox.START, etk.VBox.EXPAND_FILL, 0)

###### Mask Page ######
def clear(e):
	print "Text was: %s" % e.text

def click(b,e):
	clear(e)

mask_box = etk.VBox()
entry = etk.Entry()
entry.text = "Password"
entry.password_mode_set(True)
entry.on_text_activated(clear)

bye = etk.Button(label="Click")
bye.on_clicked(click, entry)

mask_box.append(entry, etk.VBox.START, etk.VBox.EXPAND_FILL,0)
mask_box.append(bye, etk.VBox.END, etk.VBox.NONE, 0)

###### System Information Page #####
def fetch_info():
	info = open("/etc/om-version").readlines()
	count=1
	for line in info:
		if count == 2:
			networks = str(line.strip())
		count+=1
	return str(networks)

sysinfo_box = etk.VBox()
sysinfo_label = etk.Label(text=fetch_info(), alignment=(0.5,0.5))

sysinfo_box.append(sysinfo_label, etk.VBox.START, etk.VBox.EXPAND_FILL, 0)

###### WiFi Page ######

def text_clicked(row):
    print "Clicked in item", row

def label_text_wifi(ap):
    return ITEMFONTSIZE + ap.essid_get() +"<br>" + SUBFONTSIZE \
+ "Quality: "+ap.quality_get() +"%"

def update_status(lab):
    ans= LABELFONTSIZE + "<b>device:</b>   "+wifi.device+"<br>"
    ans+="<b>Access Point:</b>   "+wifi.APmac+"<br>"
    ans+="<b>Essid:</b>   " + wifi.essid_get() + "<br>"
    ans+="<b>IP:</b>   " + wifi.ip_get() + "<br>"
    if wifi.connection_check():
        ans += "<b>Connected!!</b>"
    else:
        ans += "<b>Not Connected Yet!!</b>"
    lab.text = ans
    return 1

def connect_to_AP_confirm(AP,lst):
    # for now always answer yes
    return True

def kill_udhcpc_timer():
    os.system("killall udhcpc");
    return 0

def connect_to_AP(AP):
    print "Connecting to "+AP.name
    wifi.essid_set(AP.name);
    print "Trying to get IP"
    wifi.dhcp()
    ecore.timer_add(60,kill_udhcpc_timer)
    update_status(lab)

def connect_button_clicked(b):
    print "button clicked "
    r = lst_wifi.selected_rows
    if r:
        AP=r[0][0]
        print "Try to action label " + r[0][1]
        #connect_to_AP_confirm(AP,lst)
        connect_to_AP(AP)

def update_list(lst,wifi):
    AP=wifi.scan_ap()
    for row in lst.rows:
        row[2] = row[2]-1;
    for ap in AP:
        found=0
        for row in lst.rows:
            if ap.name == row[0].name:
                row[0]=ap
                row[1]=label_text_wifi(ap)
                found=1
        if found==0 and ap.openness_get():
            lst.freeze()
            model_wifi.append([ap, label_text_wifi(ap), 5]);
            lst.thaw()
    for row in lst.rows:
        if row[2] <= 0:
            lst.freeze()
            lst.rows.remove(row);
            lst.thaw()
    return 1

def init_gui():
    bottom_box = etk.HBox();
    connect_button = etk.Button(label="Connect");
    bottom_box.append(connect_button, etk.HBox.START , etk.HBox.EXPAND_FILL, 0)
    model_wifi = etk.ListModel()
    ktr_wifi = etk.KineticTextRenderer(slot=1, click=text_clicked)
    lst_wifi = etk.List(model_wifi, selectable=True,
animated_changes=True,row_height=60, columns=[(480,ktr_wifi,True)])
    lab = etk.Label(text="Waiting for updating");
    main_box_wifi = etk.VBox();
    main_box_wifi.append(lab,etk.VBox.START, etk.VBox.FILL, 0);
    main_box_wifi.append(lst_wifi,etk.VBox.START, etk.VBox.EXPAND_FILL, 0);
    main_box_wifi.append(bottom_box,etk.VBox.START, etk.VBox.FILL, 0);
    return connect_button, lst_wifi, model_wifi, lab, main_box_wifi;

##### Display Page ############

display_box = etk.VBox()
dp_model = etk.ListModel()
dp_ktr = etk.KineticTextRenderer(slot=1, click=clicked)
dp_lst = etk.List(dp_model, selectable=True,
animated_changes=True,row_height=80,
columns=[(480,dp_ktr,True)])

for item in ['Brightness']:
        dp_lst.freeze()
        dp_model.append([item, label_text(item)])
        dp_lst.thaw()
display_box.append(dp_lst, etk.VBox.START, etk.VBox.EXPAND_FILL, 0)

###############################################

connect_button, lst_wifi, model_wifi, lab, main_box_wifi= init_gui()
if sys.argv.__len__() >= 2 and sys.argv[1] :
    device=sys.argv[1]
else:
    device="eth0"
wifi = wifi(device)

connect_button.on_clicked(connect_button_clicked);
back=etk.Button(label="Back")
back.on_clicked(click_back)
main_box_wifi.append(back,etk.VBox.START,etk.VBox.FILL,0)
back1=etk.Button(label="Back")
back1.on_clicked(click_back1)
display_box.append(back1,etk.VBox.START,etk.VBox.FILL,0)
back2=etk.Button(label="Back")
back2.on_clicked(click_back2)
sysinfo_box.append(back2,etk.VBox.START,etk.VBox.FILL,0)

embed1.add(main_box)
embed1.show_all()
edj.part_swallow("swallow1", embed1.object)

embed2.add(display_box)
embed2.show_all()
edj.part_swallow("swallow2", embed2.object)

embed3.add(main_box_wifi)
embed3.show_all()
edj.part_swallow("swallow3", embed3.object)

embed4.add(sysinfo_box)
embed4.show_all()
edj.part_swallow("swallow4", embed4.object)

ee.show()
ecore.main_loop_begin()
